 <?php

function benchmark($callback, $parameters = array(), $verbose = true)
{
   $time_start = microtime(true);
   $parameters = array_merge($parameters, array('verbose' => $verbose));

   call_user_func_array($callback, $parameters);

   $time_end = microtime(true);
   $time = $time_end - $time_start;
   if ($verbose) echo sprintf("Durée : %s secondes\n\n",$time);

   return $time;
}


function testSQLRaw($db_host, $db_user, $db_password, $db_name,$db_port, $verbose = true)
{
    $mysqli = new mysqli($db_host, $db_user, $db_password, $db_name,$db_port);
    /* V�rification de la connexion */
    if (mysqli_connect_errno()) {
       echo 'Connexion échouée : ' . mysqli_connect_error() . "\n";
       exit();
    }
    $query = "SELECT * FROM station WHERE name = 'MY STATION' AND city LIKE '%MA CITY%'";

    if ($result = $mysqli->query($query)) {
       $result->close();
    }
    $mysqli->close();
}

function testSQLPreparedStatement($db_host, $db_user, $db_password, $db_name, $db_port, $verbose = true)
{
    $dsn = sprintf('mysql:dbname=%s;host=%s;port=%d',$db_name, $db_host, $db_port);

    try {

       $dbh = new PDO($dsn, $db_user, $db_password);
       $query = "SELECT * FROM station WHERE name = :name AND city LIKE :city";
       $statement = $dbh->prepare($query);

       $a_param = array(
           ':name' => "MY STATION",
           ':city' => "%MA CITY%",
       );

       foreach ($a_param as $key => $value)
       {
           $statement->bindValue($key, $value);
       }
       $result = $statement->execute();

       if (true === $result)
       {
           if ($verbose) echo sprintf("Il y a %d lignes !\n",$statement->rowCount());
           $statement->closeCursor();
       }
       else
       {
           echo sprintf("\nErreur durant le traitement de la query suivante :\n%s\n%s\n", $statement->queryString, implode("\n",$statement->errorInfo()));
       }
    } catch (PDOException $e) {
       echo 'Connexion échouée : ' . $e->getMessage();
    }
}

function displayResultRawVSPreparedStatement($duree_raw_sql, $duree_prepared_statement)
{
    if ($duree_raw_sql == $duree_prepared_statement)
    {
       echo "Egalité entre la durée des deux requ�tes\n";
    }

    elseif ($duree_raw_sql < $duree_prepared_statement)
    {
       echo sprintf("La requête préparée dure %f secondes plus longtemps !\n", ($duree_prepared_statement-$duree_raw_sql));
    }

    else
    {
       echo sprintf("La requête préparée dure %f secondes moins longtemps !\n", ($duree_raw_sql - $duree_prepared_statement));
    }
}

function displayRatioWithRawForReference($duree_raw_sql, $duree_prepared_statement)
{
    $ratio = 100-(($duree_raw_sql*100)/$duree_prepared_statement);
    echo sprintf("Ecart (en pourcentage) entre la durée de la PS et la Raw : %f%s!\n", $ratio,'%');
}

$db_name = 'mydatabase';
$db_host = 'localhost';
$db_port = 3306;
$db_user = 'myusername';
$db_password = 'mypassword';

echo "Execution de la requête SQL Raw :\n";
$duree1 = benchmark('testSQLRaw',array($db_host, $db_user, $db_password, $db_name, $db_port));

echo "Execution de la requête SQL préparée :\n";
$duree2 = benchmark('testSQLPreparedStatement',array($db_host, $db_user, $db_password, $db_name, $db_port));

echo "- Résultat pour une exécution unique :\n";
displayRatioWithRawForReference($duree1,$duree2);
displayResultRawVSPreparedStatement($duree1,$duree2);

$nb_iteration = 10000;
$total_duree1 = 0;
$total_duree2 = 0;

for ($i=0;$i<$nb_iteration;$i++)
{
    $total_duree1 += benchmark('testSQLRaw',array($db_host, $db_user, $db_password, $db_name, $db_port), false);
    $total_duree2 += benchmark('testSQLPreparedStatement',array($db_host, $db_user, $db_password, $db_name, $db_port), false);
}

// Moyenne du la durée d'execution de la requête pour X itérations
$duree1 = $total_duree1/$nb_iteration;
$duree2 = $total_duree2/$nb_iteration;

echo sprintf("- Moyenne des résultats pour %d exécution(s) :\n", $nb_iteration);
echo sprintf("Temps d'exécution total (SQL Raw) : %f\n",$total_duree1);
echo sprintf("Temps d'exécution total (PS) : %f\n",$total_duree2);
displayRatioWithRawForReference($total_duree1,$total_duree2);
displayResultRawVSPreparedStatement($duree1,$duree2);
?> 
